﻿// © 2016 Sitecore Corporation A/S. All rights reserved.

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Sitecore.Analytics;
using Sitecore.Data;
using Sitecore.Diagnostics;
using Sitecore.EmailCampaign.Model.Exceptions;
using Sitecore.ExM.Framework.Diagnostics;
using Sitecore.Modules.EmailCampaign.Core;
using Sitecore.Modules.EmailCampaign.Factories;
using Sitecore.Modules.EmailCampaign.Recipients;
using Sitecore.Modules.EmailCampaign.Validators;
using Sitecore.Modules.EmailCampaign.Xdb;
using Sitecore.Resources;
using Sitecore.StringExtensions;
using Sitecore.Web.UI.WebControls;

namespace Sitecore.Modules.EmailCampaign.Layouts
{
    public class SubscriptionForm : UserControl
    {
        /// <summary>
        /// The logger.
        /// </summary>
        [NotNull]
        private readonly ILogger _logger;

        /// <summary>
        /// The email RegEx validator.
        /// </summary>
        [NotNull]
        private readonly RegexValidator _emailRegexValidator;

        /// <summary>
        /// The client API.
        /// </summary>
        [NotNull]
        private readonly ClientApiBase _clientApi;

        /// <summary>
        /// The analytics tracker.
        /// </summary>
        private readonly ITracker _tracker;

        private string _rootId;
        private RecipientRepository _recipientRepository;

        protected bool ShowList;
        protected bool ConfirmSubscription = true;
        protected string RawIncludeRecipientLists;

        protected Button Confirm;
        protected Button SubscribeBtn;
        protected TextBox Email;
        protected Label EmailLabel;
        protected ImageButton SubscribeImg;
        protected HtmlGenericControl EmailArea;
        protected HtmlGenericControl ListsArea;
        protected HtmlGenericControl ListsHeader;
        protected HtmlGenericControl SrcList;
        protected HtmlGenericControl Warning;

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriptionForm"/> class. 
        /// Initializes a new instance of the <see cref="T:System.Web.UI.UserControl"/> class.
        /// </summary>
        public SubscriptionForm()
            : this(Logger.Instance)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriptionForm"/> class.
        /// Initializes a new instance of the <see cref="T:System.Web.UI.UserControl"/> class.
        /// </summary>
        /// <param name="logger">The <see cref="ILogger"/>.</param>
        public SubscriptionForm([NotNull] ILogger logger)
            : this(logger, (RegexValidator)Configuration.Factory.CreateObject("emailRegexValidator", true))
        {
            Assert.ArgumentNotNull(logger, "logger");

            _logger = logger;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriptionForm"/> class.
        /// Initializes a new instance of the <see cref="T:System.Web.UI.UserControl"/> class.
        /// </summary>
        /// <param name="logger">The <see cref="ILogger"/>.</param>
        /// <param name="emailRegexValidator">The <see cref="RegexValidator"/>.</param>
        public SubscriptionForm([NotNull] ILogger logger, [NotNull] RegexValidator emailRegexValidator)
            : this(logger, emailRegexValidator, Factory.GetClientApiInstance(), Tracker.Current)
        {
            Assert.ArgumentNotNull(logger, "logger");
            Assert.ArgumentNotNull(emailRegexValidator, "emailRegexValidator");

            _logger = logger;
            _emailRegexValidator = emailRegexValidator;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriptionForm"/> class.
        /// Initializes a new instance of the <see cref="T:System.Web.UI.UserControl"/> class.
        /// </summary>
        /// <param name="logger">The <see cref="ILogger"/>.</param>
        /// <param name="emailRegexValidator">The <see cref="RegexValidator"/>.</param>
        /// <param name="clientApi">The <see cref="ClientApiBase"/>.</param>
        /// <param name="tracker">The <see cref="ITracker"/>.</param>
        public SubscriptionForm([NotNull] ILogger logger, [NotNull] RegexValidator emailRegexValidator, [NotNull] ClientApiBase clientApi, ITracker tracker)
        {
            Assert.ArgumentNotNull(logger, "logger");
            Assert.ArgumentNotNull(emailRegexValidator, "emailRegexValidator");
            Assert.ArgumentNotNull(clientApi, "clientApi");

            _logger = logger;
            _emailRegexValidator = emailRegexValidator;
            _clientApi = clientApi;
            _tracker = tracker;
        }

        protected virtual string RecipientEmail
        {
            get
            {
                var recipient = _recipientRepository.GetRecipientSpecific(RecipientId, typeof(Email));

                if (recipient == null)
                {
                    return null;
                }

                var email = recipient.GetProperties<Email>().DefaultProperty;

                if (email == null)
                {
                    return null;
                }

                return email.EmailAddress;
            }
        }

        protected virtual RecipientId RecipientId
        {
            get
            {
                RecipientId recipientId = null;

                var contactId = ContactId;

                if (contactId != (ID)null)
                {
                    recipientId = new XdbContactId(contactId);
                }

                return recipientId;
            }
        }

        protected virtual ID ContactId
        {
            get
            {
                if (Email.Visible && !string.IsNullOrEmpty(Email.Text))
                {
                    var anonymousId = _clientApi.GetAnonymousIdFromEmail(Email.Text);

                    if (anonymousId.HasValue)
                    {
                        return new ID(anonymousId.Value);
                    }
                }

                return _tracker != null ? new ID(_tracker.Contact.ContactId) : null;
            }
        }

        protected virtual SubscriptionInfo[] SubscriptionInfos
        {
            get
            {
                if (RecipientId == null || string.IsNullOrEmpty(RawIncludeRecipientLists))
                {
                    return null;
                }

                var contactListIds = Data.ID.ParseArray(RawIncludeRecipientLists).Select(l => l.ToString()).ToArray();

                return _clientApi.GetSubscriptionInfo(RecipientId, contactListIds);
            }
        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            _recipientRepository = EcmFactory.GetDefaultFactory().Bl.RecipientRepository;

            var parameters = GetSublayoutParams();

            var val = parameters["Show List"];
            ShowList = (val != null) && val == "1";

            val = parameters["Recipient Lists"];
            RawIncludeRecipientLists = val ?? string.Empty;

            val = parameters["Manager root"];
            _rootId = val ?? string.Empty;

            if (SubscriptionInfos != null && ShowList)
            {
                foreach (var info in SubscriptionInfos)
                {
                    AddSrcComboBox(info);
                }
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Warning.InnerText = string.Empty;

            if (!IsPostBack)
            {
                EmailLabel.Text = Util.GetFrontEndText("email address");
                SubscribeBtn.Text = Util.GetFrontEndText("subscribe");
                ListsHeader.InnerText = Util.GetFrontEndText("newsletters");

                if (SubscriptionInfos != null && SubscriptionInfos.Any())
                {
                    Email.Text = RecipientEmail ?? string.Empty;

                    InitializeTextBox(Email, "enter email");

                    if (!ShowList)
                    {
                        ListsArea.Visible = false;
                        EmailArea.Attributes["class"] = "area";
                        EmailLabel.CssClass = "title";
                        EmailLabel.Text = Util.GetFrontEndText("simple title");
                    }
                }
                else
                {
                    ListsArea.Visible = false;
                    Warning.InnerText = Util.GetFrontEndText("no newsletters");
                }


                SubscribeImg.ImageUrl = Images.GetThemedImageSource(SubscribeImg.ImageUrl);

                EmailLabel.Visible = Email.Visible;
                SubscribeBtn.Visible = ShowList && Email.Visible;
                SubscribeImg.Visible = !ShowList && Email.Visible;
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            if (Request.QueryString["sc_duration"] == "temporary")
            {
                // If we're in Review mode, exit early as registering Js and Css in the html body causes validation errors
                return;
            }

            var key = "emailcampaign_js";
            ScriptManager.RegisterClientScriptInclude(this, typeof(Page), key, ResolveUrl("~") + "EmailCampaign.js");

            key = "emailcampaign_css";
            var script = "<link href='" + ResolveUrl("~") + "emailcampaign.css' rel='stylesheet' type='text/css' />";
            if (ScriptManager.GetCurrent(Page) == null || !IsPostBack)
            {
                ScriptManager.RegisterClientScriptBlock(this, typeof(Page), key, script, false);
            }
        }

        protected virtual void InitializeTextBox(TextBox control, string helpText)
        {
            control.Visible = true;

            helpText = Util.GetFrontEndText(helpText);

            if (string.IsNullOrEmpty(control.Text))
            {
                control.Text = helpText;
            }

            control.Attributes.Add("onkeydown", "if (event.keyCode == 13) {document.getElementById('" + SubscribeBtn.ClientID + "').click();} return (event.keyCode != 13);");
            control.Attributes.Add("onfocus", "textbox_onfocus('{0}', '{1}')".FormatWith(control.ClientID, helpText));
            control.Attributes.Add("onblur", "textbox_onblur('{0}', '{1}')".FormatWith(control.ClientID, helpText));
        }

        protected void AddConfirmationDlg(string text)
        {
            var script = "<script type=\"text/javascript\">var text='" + text + "'; setTimeout(\"if(confirm(text)) document.getElementById('" + Confirm.ClientID + "').click();\", 0);</" + "script>";
            SubscribeBtn.Parent.Controls.Add(new LiteralControl(script));
        }

        protected virtual void Confirm_click(object sender, EventArgs args)
        {
            try
            {
                var email = Email.Text;

                if (!string.IsNullOrEmpty(RecipientEmail) && !email.Equals(RecipientEmail, StringComparison.OrdinalIgnoreCase))
                {
                    UpdateEmailInXDB();
                }

                Subscribe_click(SubscribeBtn, new EventArgs());
            }
            catch (EmailCampaignException e)
            {
                Warning.InnerText = e.LocalizedMessage;
            }
            catch (ProviderException)
            {
                Warning.InnerText = Util.GetFrontEndText("email in use");
            }
            catch (Exception e)
            {
                Warning.InnerText = e.Message;
            }
        }

        protected virtual NameValueCollection GetSublayoutParams()
        {
            var parameters = new NameValueCollection();

            if (!(Parent is Sublayout))
            {
                return parameters;
            }

            var paramsStr = HttpUtility.UrlDecode((Parent as Sublayout).Parameters);

            foreach (var param in paramsStr.Split('&'))
            {
                var parts = param.Split('=');
                parameters.Add(parts[0], parts.Length == 1 ? string.Empty : parts[1]);
            }

            return parameters;
        }

        /// <exception cref="EmailCampaignException"><c>EmailCampaignException</c>.</exception>
        protected virtual void Subscribe_click(object sender, EventArgs args)
        {
            try
            {
                if (!string.IsNullOrEmpty(RecipientEmail) && !Email.Text.Equals(RecipientEmail, StringComparison.OrdinalIgnoreCase))
                {
                    if (!_emailRegexValidator.IsValid(Email.Text))
                    {
                        throw new EmailCampaignException(Util.GetFrontEndText("email not valid"));
                    }

                    var text = Util.GetFrontEndText("address changed confirmation");
                    AddConfirmationDlg(string.Format(text, Util.GetFrontEndText("an email")));
                    return;
                }

                if (string.IsNullOrEmpty(RecipientEmail))
                {
                    UpdateEmailInXDB();
                }

                var listsToSubscribe = new List<string>();
                var listsToUnsubscribe = new List<string>();

                if (ShowList)
                {
                    var items = GetSrcListItems();

                    for (var i = 0; i < items.Count; i++)
                    {
                        var checkBox = items[i];
                        Guid recipientListId;
                        if (!Guid.TryParse(checkBox.Attributes["value"], out recipientListId))
                        {
                            continue;
                        }

                        if (checkBox.Checked)
                        {
                            listsToSubscribe.Add(recipientListId.ToString());
                        }
                        else
                        {
                            var listId = new ID(recipientListId).ToString();
                            var info = SubscriptionInfos
                                .Cast<SubscriptionInfo?>()
                                .FirstOrDefault(x => x.HasValue && x.Value.ContactListId == listId && x.Value.UserSubscribed);

                            if (info != null)
                            {
                                listsToUnsubscribe.Add(listId);
                            }
                        }
                    }
                }
                else
                {
                    for (var i = 0; i < SubscriptionInfos.Length; i++)
                    {
                        listsToSubscribe.Add(SubscriptionInfos[i].ContactListId);
                    }
                }

                var itemIdToRedirect = _clientApi.UpdateSubscriptions(RecipientId, listsToSubscribe.ToArray(), listsToUnsubscribe.ToArray(), _rootId, ConfirmSubscription);

                if (!string.IsNullOrEmpty(itemIdToRedirect))
                {
                    var url = ItemUtilExt.GetContentItemPageUrl(itemIdToRedirect);
                    if (!string.IsNullOrEmpty(url))
                    {
                        Response.Redirect(url, false);
                        Context.ApplicationInstance.CompleteRequest();
                    }
                }
            }
            catch (EmailCampaignException e)
            {
                Warning.InnerText = e.LocalizedMessage;
            }
            catch (Exception e)
            {
                Warning.InnerText = Util.GetFrontEndText(e.Message);
                _logger.LogError(e);
            }
        }

        protected virtual void UpdateEmailInXDB()
        {
            _recipientRepository.UpdateRecipientEmail(RecipientId, Email.Text);
        }

        private void AddSrcComboBox(SubscriptionInfo info)
        {
            var checkBox = new CheckBox();

            checkBox.Attributes["value"] = info.ContactListId;
            checkBox.AutoPostBack = true;
            checkBox.CssClass = "msCheckBox";
            checkBox.Checked = info.UserSubscribed;

            checkBox.Text = info.Name;

            SrcList.Controls.Add(checkBox);

            if (!String.IsNullOrEmpty(info.Description))
            {
                SrcList.Controls.Add(new Literal
                {
                    Text = "<div class=\"msDescription\">" + info.Description + "</div>"
                });
            }
        }

        private List<CheckBox> GetSrcListItems()
        {
            return SrcList.Controls.OfType<CheckBox>().ToList();
        }
    }
}

